// --8<--8<--8<--8<--
//
// Copyright (C) 2013 Smithsonian Astrophysical Observatory
//
// This file is part of transform_ray
//
// transform_ray is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

// process.cc

#include <rl_raylib/rl_Ray.h>                // rl_Ray
#include <tracefctxx/TraceFct.h>

#include "param.h"
#include "rot_BPipe.h"             // rot_BPipe

#include "process.h"

void
process( Pars& pars) {

  TraceFct tf("process");

  rot_BPipe bp( pars.input, pars.output );

  rl_Ray      ray;

  dvm3_RotMat null_rotate;
  dvm3_Vector null_translate( 0, 0, 0 );

  size_t nphot;

  for ( nphot = 0;  bp.read( ray ) ; ++nphot ) {

      // all of the input transformations are generated via an affine
      // transformation matrix, which effectively applies a rotation
      // followed by a translation.  Unfortunately, rl_raylib only
      // provides a single translate then rotate function.  Rather
      // than hack rl_raylib and all of its ancestors, separate the
      // two operations.

      ray.translate_rotate( null_translate, pars.rot );
      ray.translate_rotate( pars.off, null_rotate );

      if ( bp.write( ray ) )
	  tf.die( "error writing photon %d: %s", nphot,
		  bpipe_strerror(bpipe_errno ));

  }

  if ( bpipe_errno != BPNOERROR )
      tf.die( "error reading photon %d: %s", nphot + 1,
	      bpipe_strerror( bpipe_errno) );


}
