// --8<--8<--8<--8<--
//
// Copyright (C) 2013 Smithsonian Astrophysical Observatory
//
// This file is part of transform_ray
//
// transform_ray is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#ifndef rot_BPipe_h_INCLUDED
#define rot_BPipe_h_INCLUDED

/****************************************************************************
 * Author     : Terry Gaetz
 *
 * Description: handle bpipe stuff.
 */

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>                    // misc. #defines

#include <rl_raylib/rl_Ray.h>

//########################################################################
// rot_BPipe
//########################################################################
//
//: Manage a ray bpipe.

class rot_BPipe
{
private:

  void*        bpipe;            // BPipe*
  void*        bpo;              // BPipeOutput*
  void*        data;             // BPipe data

  long         n_ray;            // number of rays read

  int          have_polstate;    // true if polarization fields are present

  rot_BPipe();                               // PROHIBITED
  rot_BPipe( rot_BPipe const& );             // PROHIBITED
  rot_BPipe& operator=( const rot_BPipe & );  // PROHIBITED

public:

  struct
  {
      DVector3     *pos;    // ray position
      DVector3     *dir;    // ray direction vector (direction cosines)
      double       *energy; // photon energy
      DCVector2    *c2;     // polarization amplitude (cosine component)
      DCVector2    *s2;     // polarization amplitude (sine component)
  } raydata;

 ~rot_BPipe();
    //
    //: Destructor.
    //  Close down the bpipe.

  rot_BPipe( const string& instream, const string& outstream );

  void init( const string& instream, const string& outstream );

  int read(  rl_Ray& ray );

  int write( rl_Ray const& ray );

  int have_polarization_state() const { return have_polstate; }


};


#endif /* rot_BPipe_h_INCLUDED */
